@@ -66,17 +66,18 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
 
     let gen = quote! {
         mod checkout_server {
+            pub use std::sync::Arc;
             pub use checkout_core::dropshot::ApiDescription;
             pub use checkout_core::dropshot::ConfigDropshot;
             pub use checkout_core::dropshot::ConfigLogging;
             pub use checkout_core::dropshot::ConfigLoggingLevel;
             pub use checkout_core::dropshot::HttpServerStarter;
             pub use checkout_core::error::Error;
+            pub use checkout_core::error::new_server_error;
             pub use checkout_core::server::Server;
             pub use checkout_core::internal_context::InternalContext;
 
             use std::env;
-            use std::sync::Arc;
             use checkout_core::dropshot;
             use checkout_core::dropshot::endpoint;
             use checkout_core::dropshot::RequestContext;
@@ -103,6 +104,7 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                 pub google_application_credentials: String,
                 pub payment_processing_topic: String,
                 pub checkout_completion_topic: String,
+                pub bind_address: String,
             }
 
             pub fn get_config() -> Config {
@@ -110,12 +112,13 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                     google_application_credentials: env::var("GOOGLE_APPLICATION_CREDENTIALS").unwrap(),
                     payment_processing_topic: env::var("PAYMENT_PROCESSING_TOPIC").unwrap(),
                     checkout_completion_topic: env::var("CHECKOUT_COMPLETION_TOPIC").unwrap(),
+                    bind_address: env::var("BIND_ADDRESS").unwrap(),
                 }
             }
 
             pub struct ServerContext {
                 pub internal_context: InternalContext,
-                pub context_provider: super::#server_type_name,
+                pub context_provider: Arc<super::#server_type_name>,
             }
 
             #[endpoint {
@@ -422,47 +425,39 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                 api
             }
 
-            async fn new_server(
-                self,
-                bind_address: &str,
-            ) -> Result<
-                checkout_server::HttpServerStarter<checkout_server::ServerContext>,
-                checkout_server::Error,
-            > {
+            async fn start(
+                self
+            ) -> Result<(), checkout_server::Error> {
                 let log = checkout_server::ConfigLogging::StderrTerminal {
                     level: checkout_server::ConfigLoggingLevel::Info,
                 }
                 .to_logger("minimal-example")
-                .map_err(|_| checkout_server::Error {
-                    code: "500".to_string(),
-                    message: "failed to bind server".to_string(),
-                })?;
+                .map_err(|_| checkout_server::new_server_error("failed to setup logger"))?;
 
                 let api = Self::api_description();
 
                 let config = checkout_server::get_config();
 
-                let internal_context = checkout_server::InternalContext::new(config.google_application_credentials.clone(), config.payment_processing_topic.clone(), config.checkout_completion_topic.clone()).await.map_err(|_| checkout_server::Error {
-                    code: "500".to_string(),
-                    message: "failed to create internal context".to_string(),
-                })?;
+                let internal_context = checkout_server::InternalContext::new(config.google_application_credentials.clone(), config.payment_processing_topic.clone(), config.checkout_completion_topic.clone()).await.map_err(|_| checkout_server::new_server_error("failed to create internal context"))?;
+
+                let context_provider = checkout_server::Arc::new(self);
 
                 checkout_server::HttpServerStarter::new(
                     &checkout_server::ConfigDropshot {
-                        bind_address: bind_address.parse().unwrap(),
+                        bind_address: config.bind_address.parse().unwrap(),
                         request_body_max_bytes: 2048 * 1024,
                     },
                     api,
                     checkout_server::ServerContext {
                         internal_context,
-                        context_provider: self,
+                        context_provider: context_provider.clone(),
                     },
                     &log,
                 )
-                .map_err(|_| checkout_server::Error {
-                    code: "500".to_string(),
-                    message: "failed to bind server".to_string(),
-                })
+                .map_err(|_| checkout_server::new_server_error("failed to set up api server"))?
+                .start()
+                .await
+                .map_err(|_| checkout_server::new_server_error("failed to start server"))
             }
         }
     };
